package com.yuandian.dataflow.statemachine;

import java.io.File;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.concurrent.Executor;
import java.util.concurrent.atomic.AtomicLong;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.alipay.remoting.exception.CodecException;
import com.alipay.remoting.serialization.SerializerManager;
import com.alipay.sofa.jraft.Closure;
import com.alipay.sofa.jraft.Iterator;
import com.alipay.sofa.jraft.Status;
import com.alipay.sofa.jraft.core.StateMachineAdapter;
import com.alipay.sofa.jraft.entity.PeerId;
import com.alipay.sofa.jraft.error.RaftError;
import com.alipay.sofa.jraft.error.RaftException;
import com.alipay.sofa.jraft.error.RemotingException;
import com.alipay.sofa.jraft.option.CliOptions;
import com.alipay.sofa.jraft.rpc.InvokeCallback;
import com.alipay.sofa.jraft.rpc.impl.BoltRaftRpcFactory;
import com.alipay.sofa.jraft.storage.snapshot.SnapshotReader;
import com.alipay.sofa.jraft.storage.snapshot.SnapshotWriter;
import com.alipay.sofa.jraft.util.Endpoint;
import com.alipay.sofa.jraft.util.Utils;
import com.yuandian.dataflow.statemachine.rpc.SMResponse;
import com.yuandian.dataflow.statemachine.rpc.SyncData;
import com.yuandian.dataflow.statemachine.rpc.TaskState;

import lombok.var;
import lombok.extern.slf4j.Slf4j;

/**
 * Counter state machine.
 *
 * @author boyan (boyan@alibaba-inc.com)
 *
 *         2018-Apr-09 4:52:31 PM
 */
@Slf4j

public class StateMachine extends StateMachineAdapter {

    // private static final Logger LOG = LoggerFactory.getLogger(StateMachine.class);

    /**
     * Counter value
     */
    private TaskState taskState = new TaskState();
    /**
     * Leader term
     */
    private final AtomicLong leaderTerm = new AtomicLong(-1);

    public boolean isLeader() {
        return this.leaderTerm.get() > 0;
    }

    /**
     * Returns current value.
     */
    public TaskState getTaskState() {
        return taskState;
    }

    @Override
    public void onApply(final Iterator iter) {
        while (iter.hasNext()) {

            if (iter.done() != null) {
                // This task is applied by this node, get value from closure to avoid additional
                // parsing.
                var closure = (SyncDataClosure)iter.done();
                taskState = closure.getTaskState();
                log.info("SyncDataClosure(done) taskState:{} leaderTerm:{}",taskState, this.leaderTerm); 
                closure.success(taskState);
                closure.run(Status.OK());
            } else {
                 // Have to parse FetchAddRequest from this user log.
                final ByteBuffer data = iter.getData();
                try {
                    taskState = SerializerManager.getSerializer(SerializerManager.Hessian2).deserialize(
                        data.array(), TaskState.class.getName());
                        log.info("SyncDataClosure(null) taskState:{} leaderTerm:{}", taskState, this.leaderTerm);
                } catch (CodecException e) {
                    e.printStackTrace();
                }
               
            }

            iter.next();
        }
    }

    @Override
    public void onSnapshotSave(final SnapshotWriter writer, final Closure done) {
        return;
    }

    @Override
    public void onError(final RaftException e) {
        log.error("Raft error: {}", e, e);
    }

    @Override
    public boolean onSnapshotLoad(final SnapshotReader reader) {

        return true;
    }

    @Override
    public void onLeaderStart(final long term) {
        this.leaderTerm.set(term);
        super.onLeaderStart(term);

    }

    @Override
    public void onLeaderStop(final Status status) {
        this.leaderTerm.set(-1);
        super.onLeaderStop(status);
    }

    public static void main(String[] args) throws InterruptedException, RemotingException {
        
    }

}
